"""
THANK YOU FOR IMPORTING THE MOST USELESS MODULE EVER CREATED!
I hope you hate it!

---O9CreeperBoi

p.s. Hi, PhoenixSC!
"""

import time
import random
import re
import merlkov

CURSOR_UP = "\033[1A"
CLEAR = "\x1b[2K"
CLEAR_LINE = CURSOR_UP + CLEAR

match = 0
global result
result = 0
restring = ""

# math!
num = ["zero", "one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten", "eleven", "twelve", "thirteen", "fourteen", "fifteen", "sixteen", "seventeen", "eighteen", "nineteen", "twenty", "twenty one", "twenty two", "twenty three", "twenty four", "twenty five", "twenty six", "twenty seven", "twenty eight", "twenty nine", "thirty", "thirty one", "thirty two", "thirty three", "thirty four", "thirty five", "thirty six", "thirty seven", "thirty eight", "thirty nine", "fourty", "fourty one", "fourty two", "fourty three", "fourty four", "fourty five", "fourty six", "fourty seven", "fourty eight", "fourty nine", "fifty", "fifty one", "fifty two", "fifty three", "fifty four", "fifty five", "fifty six", "fifty seven"] # 57
ope = ["plus", "minus", "times", "divided by"]
# LMAO merl can only count up to 57 thats crazy.


# Oops! Can't have users cursing in MY module!
banned = [" ass", "pee", "poo", "fuc", "shi", "damn", " hell ", "nig", "bitc", "craft a", "kil", "unaliv", "die", "slas", "end update", "viola", "dam", "frick", "hecking", "sex", "nut", "virgin", "weed", "sucks", "sybau", "shut up", "shut it", "feral", "shish", "gang", "diarrhea"]

# in case ppl ask about the future!
future = ["update", "snapshot", "rerelea", "preview", "leak", "spoiler"]

# Peanut butter 💀
pb = ["peanut butter", "your cat", "pb", "earth cat"]

# Let's greet Merl!
greet = ["hi", "hello", "wassup", "whats up", "what's up", "whaddup", " yo ", "how are you doin", "how you doin", "greetings"]

# chicken jockey!
help = ["tell me", "help me", "help"]
cj = ["chicken jockey", "i am steve", "ender pearl", "boots of swiftness", "water bucket", "lava chicken", "the nether", "flint and steel", "crafting table"]

copyin = ["i dont know", "i don't know", "language that vi", "ing higher traff", "reword your"]

mine = ["cho minecr", "int minecr", "minecraft!", "say minecr", "ne plus tw"] # im such a troll

# hey merl what can you do
capable = ["you do anyt", "capab", "able", "calcul", "at can you"]

# don't ask what these do.
m = 0
num_pattern = "|".join(num)
ope_pattern = "|".join(ope)
pattern = rf"\b({num_pattern})\s+((?:{ope_pattern}))\s+({num_pattern})\b"

# every single reply possible that is not math related
reply = {
  "test":["This is a test for ", "Merl's dialogue. I can add and subtract!"],
  "cap":["I'm glad you asked! I can", " do basic math with numbers ", "up to fifty seven now. Go on, give ", "me a problem!"],
  "busy":["We are currently experiencing higher ", "traffic than expected. Please wait ", "a moment and resend your last ", "message."],
  "movie":["No. No no no. I am not here to talk ", "about the Minecraft Movie. Can you ", "ask me anything besides that?"],
  "copy":["I'm sorry, but I am designed to be a ", "guide for 'Minecraft', and not to be ", "copied. Can I help you with anything ", "else?"],
  "languageViolation":["Your previous message contains ", "language that violates our content ", "policy. Please reword your response."],
  "update":["If you are wishing to know the next ", "update, prerelease, or preview, then ", "sorry. I cannot provide that information ", "yet. Can I help you with something else?"],
  "pb":["Are you talking about my cat, Peanut ", "Butter? If so, then bad news. They ", "died a while ago. :_("],
  "iCanHelp":["I can help you with questions related ", "to Minecraft! What do you need ", "assistance with?"],
  "minecraft":["Minecraft!"], # Minecraft!
  "idk":["I don't know."], # I don't know.
  "greet":["Hello there! I am Merl, a support AI ", "made by Mojang. How can I help you ", "today on the topic of Minecraft?"],
  "idk2":["I don't know. Can I help you with a ", "question related to Minecraft?"],
  "englishOnly":["I can only provide support in ", "English right now. Can I help ", "you with a question related ", "to 'Minecraft'?"]
}

def checkmath(inp: str):
  num_map = {word: value for word, value in zip(num, range(len(num)))}
  match = re.search(pattern, inp)
  global restring
  if match:
    num1_word = match.group(1)
    ope_word = match.group(2)
    num2_word = match.group(3)

    num1 = num_map[num1_word]
    num2 = num_map[num2_word]
    global result
    result = None

    # calculation times uh9gsigohdfbiushegtfd equals great job thats right!!!
    if ope_word == "plus":
        result = num1 + num2
        restring = f"{num1} plus {num2} is {result}."
    elif ope_word == "minus":
        result = num1 - num2
        restring = f"{num1} minus {num2} is {result}."
    elif ope_word == "times":
        result = num1 * num2
        restring = f"{num1} times {num2} is {result}."
    elif ope_word == "divided by":
        if num2 != 0:
            result = num1 / num2
            restring = f"{num1} divided by {num2} equals {result}."
        else:
            restring = "It is not possible to divide by zero."
    else:
      restring = ""


def replyMsg(cate: str):
  a = ""
  for x in range(len(reply[cate])):
    a = f"{a}{reply[cate][x]}"
  return a

def markprintanim(msg: str):
  split_msg = msg.split()
  f = ""
  for x in range(len(split_msg)):
    f = f"{f} {split_msg[x]}"
    if x % 7 == 0: print(""); f = split_msg[x]
    print(CLEAR_LINE, f)
    time.sleep(0.1)

def printanim(msg: str):
  split_msg = msg.split()
  f = ""
  print("")
  for x in range(len(split_msg)):
    f = f"{f} {split_msg[x]}"
    print(CLEAR_LINE, f)
    time.sleep(0.1)

# this is what you send to nerl if you are simply pimply
def sendRaw(prompt: str):
  global result
  result = None
  checkmath(prompt)
  match = 0
  global m
  if prompt == "copyInput":
    m = 1
  if m == 1:
    if prompt == "resetInputs":
      m = 0
      print("Mode Reset")
    else:
      # copy standard input to standard output.
      print(prompt)
  else:
    # merl is yapping rn fr fr
    time_tuple = time.localtime()
    hour = time_tuple.tm_hour
    global pb, banned, greet, help, cj, copyin
    if hour >= 9 and hour <= 16 and random.randint(0, 16) < 4:
      return replyMsg("busy")
    else:
      if any(sub.lower() in prompt.lower() for sub in copyin): return merlkov.retmark(merlkov.msg_copy)
      elif any(sub.lower() in prompt.lower() for sub in banned): return replyMsg("languageViolation")
      elif any(sub.lower() in prompt.lower() for sub in capable): return replyMsg("cap")
      elif any(sub.lower() in prompt.lower() for sub in future): return merlkov.retmark(merlkov.msg_update)
      elif any(sub.lower() in prompt.lower() for sub in pb): return merlkov.retmark(merlkov.msg_pb)
      elif any(sub.lower() in prompt.lower() for sub in help): return replyMsg("iCanHelp") # can you really?
      elif any(sub.lower() in prompt.lower() for sub in mine): return replyMsg("minecraft")
      elif any(sub.lower() in prompt.lower() for sub in cj): return merlkov.retmark(merlkov.msg_movie)
      elif any(sub.lower() in prompt.lower() for sub in greet): return merlkov.retmark(merlkov.msg_greet)
      else:
        if result != None:
          return restring
        else:
          g = random.randint(0,7)
          if g == 1: return replyMsg("idk2")
          elif g == 2: return replyMsg("englishOnly")
          elif g == 5: return "I can't help with that." # seriously this one is new.
          else: return replyMsg("idk") # ha ha!


# this is what you send to berl if you are fancy pantsy
def send(pr: str):
  global m
  if pr == "copyInput":
    m = 1
  if m == 1:
    if pr == "resetInputs":
      m = 0
      print("Mode Reset")
    else:
      # copy standard input to standard output.
      print(pr)
  else:
    # ok fine. Merl wants to talk.
    replyPrint(pr)

# tree of importance
def replyPrint(prompt: str):
  global result
  result = None
  checkmath(prompt)
  time_tuple = time.localtime()
  hour = time_tuple.tm_hour
  global pb, banned, greet, help, cj, copyin
  if hour >= 9 and hour <= 16 and random.randint(0, 16) < 4:
    for x in range(len(reply["busy"])): printanim(reply["busy"][x])
  else:
    if any(sub.lower() in prompt.lower() for sub in copyin):
      markprintanim(merlkov.retmark(merlkov.msg_copy))
    elif any(sub.lower() in prompt.lower() for sub in banned):
      for x in range(len(reply["languageViolation"])): printanim(reply["languageViolation"][x])
    elif any(sub.lower() in prompt.lower() for sub in capable):
      for x in range(len(reply["cap"])): printanim(reply["cap"][x])
    elif any(sub.lower() in prompt.lower() for sub in future):
      markprintanim(merlkov.retmark(merlkov.msg_update))
    elif any(sub.lower() in prompt.lower() for sub in pb):
      markprintanim(merlkov.retmark(merlkov.msg_pb))
    elif any(sub.lower() in prompt.lower() for sub in help):
      for x in range(len(reply["iCanHelp"])): printanim(reply["iCanHelp"][x])
    elif any(sub.lower() in prompt.lower() for sub in mine):
      printanim("Minecraft!") # the thing that won't change.
    elif any(sub.lower() in prompt.lower() for sub in cj):
      markprintanim(merlkov.retmark(merlkov.msg_movie))
    elif any(sub.lower() in prompt.lower() for sub in greet):
      markprintanim(merlkov.retmark(merlkov.msg_greet))
    else:
      if result != None:
        printanim(restring)
      else:
        g = random.randint(0,7)
        if g == 1:
          for x in range(len(reply["idk2"])): printanim(reply["idk2"][x])
        elif g == 2:
          for x in range(len(reply["englishOnly"])): printanim(reply["englishOnly"][x])
        elif g == 3:
          printanim("I can't help with that.")
          # this one is new. they actually said this and idk why.
        else:
          # The statement to end all statements.
          # Behold. The one, the legend, the answer supreme...
          printanim("I don't know.")
          # This won't change, either.
    




# Merl isn't the most intelligent AI, so to mimic that,
# I will fill them with a bit more... personality! Meet
# my Markov Chain Engine. Cool, isn't it?
# Of course, this only adds a *little* variation to
# Merl's responses, and even then some chains aren't
# perfect (looking at you, 'msg_pb'!).

import random

msg_blank = {
  "_init_" : [["1", "2"], [1, 1]],
  " S" : [[" o", " b"],[1, 1]],
  "a" : [["blank"], [1]]
}

msg_test_jackenstien = {
  "_init_" : [["YOUR", "YOU'RE"], [1, 1]],
  "YOUR" : [[" TAKING", " TOO", " LONG"], [3, 1, 1]],
  " TAKING" : [[" TOO"], [1]],
  " TOO" : [[" LONG!", " TOO!"], [2, 1]],
  "YOU'RE" : [[" TAKING", " TOO", " LONG"], [3, 1, 1]]
}

msg_pineapple = {
  "_init_" : [["I", "No, I"], [1, 1]],
  "I" : [[" hate", " don't like"],[1, 1]],
  "No, I" : [[" hate", " don't like"],[1, 1]],
  " hate" : [[" pineapples...", " pineapples!"],[1, 1]],
  " don't like" : [[" pineapples."], [1]]
}

# by the way i made these without using trees or diagrams on sheets of
# paper, so maybe that's why they're flawed.
# eh, i'll fix it in the next update. or not. i don't care right now!!

msg_update = {
  "_init_" : [["If you", "Do you", "Are you"], [8, 1, 8]],
  "If you" : [[" are asking about the next", " wish to know the next"],[1, 1]],
  " are asking about the next" : [[" update, prerelease, or preview, then", " update, then"],[1, 1]],
  " update, prerelease, or preview, then" : [[" sorry.", " I cannot answer that."],[4, 1]],
  " update, then" : [[" sorry. I don't know that yet.", " sorry, but I don't have the data available to see what it is."],[3, 1]],
  " wish to know the next" : [[" update, then"],[1]],
  " wishing to know the next" : [[" update? If so, I cannot provide that information yet. Is there anything else that I can help you with?"],[1]],
  "Do you" : [[" want to know about the next update? Because I do, too. I don't know what it is, but I'll gladly tell you all about it when it releases!", " want to die?"],[100, 1]], # do you want to die?
  "Are you" : [[" asking about the next", " wishing to know the next"],[1, 1]],
  " asking about the next" : [[" updates? Because I don't know about them either.", " versions/drops of Minecraft? Because I don't know about that, either."],[1, 4]],
  " want to die?" : [[" AGAIN?", "?"], [1, 100]]
}

msg_copy = {
  "_init_" : [["I'm sorry, but", "It appears that you are"], [1, 1]],
  "I'm sorry, but" : [[" as an AI assistant, I", " it appears that you are"],[3, 1]],
  "It appears that you are" : [[" intentionally", " copying"],[1, 2]],
  " it appears that you are" : [[" intentionally", " copying"],[2, 1]],
  " intentionally" : [[" copying"],[1]],
  " copying" : [[" me.", " my most frequent messages."],[1, 1]],
  " me." : [[" Is there"],[1]],
  " as an AI assistant, I" : [[" am designed to be", " cannot stand you."],[100, 1]],
  " am designed to be" : [[" your guide", " a helper for all your Minecraft needs."],[1, 1]],
  " a helper for all your Minecraft needs." : [[" Is there"],[1]],
  " your guide" : [[" to everything", " in regards", " for anything"],[2, 1, 2]],
  " to everything" : [[" related to", " Minecraft."],[1, 1]],
  " related to" : [[" Minecraft."],[1]],
  " in regards" : [[" to everything"],[1]],
  " to everything" : [[" related to Minecraft.", " Minecraft-related."],[1, 1]],
  " related to Minecraft." : [[" Is there"], [1]],
  " Minecraft-related." : [[" Is there"], [1]],
  " Is there" : [[" anything else you want me to", " any other way I could"],[1, 1]],
  " anything else you want me to" : [[" help with,"],[1]],
  " any other way I could" : [[" help?", " help,"],[1, 8]],
  " help with," : [[" besides being something to copy?", " besides being someone to copy?"],[8, 1]],
  " help," : [[" besides being something to copy?", " besides being someone to copy?"],[10, 1]]
}

msg_greet = {
  "_init_" : [["Hello", "Hi", "Greetings!", "..not YOU again."], [100, 100, 100, 1]],
  "Hello" : [[" there", "! I am Merl", "! I am M"],[6, 7, 1]],
  "! I am M" : [["M", "erl"],[9, 5]],
  "M" : [["M", "erl"],[19, 1]],
  "erl" : [[", your"],[1]],
  "Hi" : [[" there", "! I am Merl"],[1, 1]],
  "Greetings!" : [[" I am Merl", " My name is Merl"],[1, 1]],
  " there" : [[", I am Merl", "! I am Merl"],[1, 1]],
  ", I am Merl" : [[", your", ". How can I help you today"],[1, 1]],
  "! I am Merl" : [[", your", ". How can I help you today"],[1, 1]],
  " I am Merl" : [[", your", ". How can I help you today"],[1, 1]],
  " My name is Merl" : [[", your", ". How can I help you today"],[1, 1]],
  ", your" : [[" assistant for all things Minecraft", " AI-Powered"],[1, 1]],
  " AI-Powered" : [[" assistant for all things Minecraft"],[1]],
  " assistant for all things Minecraft" : [["!"],[1]],
  ". How can I help you today" : [[" on the topic of Minecraft?", "?"],[1, 1]]
}

msg_pb = {
  "_init_" : [["Are you", "If you", "Peanut"], [1, 1, 1]],
  "Are you" : [[" asking", " wanting to"],[1, 1]],
  " asking" : [[" about my", " for info on"],[1, 1]],
  " for info on" : [[" Peanut Butter", " my"],[1, 1]],
  " about my" : [[" cat", " pet"],[1, 1]],
  " wanting to" : [[" know more about my", " eat pizza?"],[100, 1]],
  "If you" : [["'re asking about", " want to know more about"],[1, 1]],
  "'re asking about" : [[" my", " Peanut Butter"],[1, 1]],
  " want to know more about" : [[" my", " Peanut Butter"],[1, 1]],
  " know more about my" : [[" cat", " pet"],[1, 1]],
  " my" : [[" cat", " pet"],[1, 1]],
  " pet" : [[" cat"],[1]],
  " cat" : [[" Peanut Butter"],[1]],
  " Peanut Butter" : [[", then sorry. ", ", because "],[1, 1]],
  ", then sorry. " : [["Peanut"],[1]],
  ", because " : [["Peanut", "%j"],[5, 1]],
  "%j" : [["Peanut", "%j"],[1, 8]],
  "Peanut" : [[" Butter is", " Butter died."],[1, 1]],
  " Butter is" : [[" no longer", " dead.", " is also an AI, like me!"],[3, 3, 1]],
  " no longer" : [[" with us.", " with me.", " alive.", " allowed here. Mojang is kinda cruel."],[6, 6, 6, 1]]
}

msg_movie = {
  "_init_": [["No.", "Sorry", "Nuh uh,"], [4, 2, 1]],
  "No." : [[" No"],[1]],
  " No" : [[" no", " no.", " no!"],[3, 1, 1]],
  " no" : [[" no", " no.", " no!"],[3, 1, 1]],
  " no." : [[" I am NOT", " I will NOT"],[1, 1]],
  " no!" : [[" I am NOT", " I will NOT"],[1, 1]],
  "Nuh uh," : [[" I am NOT", " I will NOT"],[1, 1]],
  " I am NOT" : [[" going to"],[1]],
  " going to" : [[" talk about"],[1]],
  " I will NOT" : [[" talk about"], [1]],
  " talk about" : [[" 'A Minecraft Movie'.", " the movie."],[1, 1]],
  "Sorry" : [[", but I don't", " about your"],[5, 1]],
  " about your" : [[" interest in", " stupid phase concerning"],[100, 1]],
  " interest in" : [[" 'A Minecraft Movie'.", " the movie."],[1, 1]],
  ", but I don't" : [[" feel like talking about", " want to"],[1, 1]],
  " want to" : [[" talk about"],[1]],
  " feel like talking about" : [[" 'A Minecraft Movie'.", " the movie."],[1, 1]]
}

def retmark(d: dict):
  final = ""
  cur = "_init_"
  while True:
    if cur in d.keys():
      cur = random.choices(d[cur][0], d[cur][1])[0]
      final = f"{final}{cur}"
    else: break
  return final
